import Ember from 'ember';
import config from 'php-api-ember-test/config/environment';

/**
 * @todo [php-api] add form validation
 * @todo [php-api] fix unable to re-submit invalid form (blank form).
 */
export default Ember.Component.extend({
    init() {
        this.set('id', this.get('editId'));
        this._super(...arguments);
    },

    actions: {
        addNew() {
            console.log('hit on submit button from add page');
            this.addNew();
        },
        editSave() {
            console.log('hit on submit button from edit page');
            this.editSave();
        }
    },

    addNew() {
        console.log('adding new data');
        var formData = new FormData($('#form-add')[0]);
        var thisobj = this;
        $.ajax({
            url: config.APP.phpApiUrl+'/add',
            type: 'POST',
            data: formData,
            processData: false, // jquery, please not process the data.
            contentType: false, // jquery, please not set content type.
            dataType: 'json',
            success: function(response) {
                console.log(response);
                if (typeof(response) !== 'undefined' && typeof(response.form_result) !== 'undefined' && typeof(response.form_result_text) !== 'undefined') {
                    thisobj.set('isAlert', true);
                    thisobj.set('alertMessage', response.form_result_text);
                    thisobj.set('alertStatus', response.form_result);
                }

                if (typeof(response) !== 'undefined') {
                    if (typeof(response.result) !== 'undefined' && typeof(response.id) !== 'undefined' && response.result === true) {
                        thisobj.sendAction('redirectTo', '/edit/'+response.id);
                    }
                }
            },
            error: function(jqXHR) {
                var response = jqXHR.responseJSON;
                console.log(response);
                if (typeof(response) !== 'undefined' && typeof(response.form_result) !== 'undefined' && typeof(response.form_result_text) !== 'undefined') {
                    thisobj.set('isAlert', true);
                    thisobj.set('alertMessage', response.form_result_text);
                    thisobj.set('alertStatus', response.form_result);
                }
            }
        });
    },

    alertStatus: '',

    alertMessage: '',

    config: config.APP,

    didReceiveAttrs() {
        console.log('did receive attributes');
        this.setDbDataToValues();
    },

    editSave() {
        console.log('saving edit data');
        if ($('#file').val() !== '' && $('#file').val() !== null) {
            this.editSaveDoUpload(this.editSaveDoUpdate);
        } else {
            this.editSaveDoUpdate(this);
        }
    },

    editSaveDoUpload(callbackFunctionAfterSuccessUpload) {
        console.log('there is a file upload, process it first');
        var inputfile = $('#file').prop('files')[0];
        var formData = new FormData();
        formData.append('file', inputfile);
        var thisobj = this;

        $.ajax({
            url: config.APP.phpApiUrl+'/editUpload/'+this.get('id'),
            type: 'POST',
            data: formData,
            processData: false, // jquery, please not process the data.
            contentType: false, // jquery, please not set content type.
            dataType: 'json',
            success: function(response) {
                console.log(response);
                if (typeof(response) !== 'undefined' && typeof(response.form_result) !== 'undefined' && typeof(response.form_result_text) !== 'undefined') {
                    thisobj.set('isAlert', true);
                    thisobj.set('alertMessage', response.form_result_text);
                    thisobj.set('alertStatus', response.form_result);
                }

                if (typeof(response) !== 'undefined') {
                    if (typeof(response.result) !== 'undefined' && response.result === true) {
                        if (typeof(callbackFunctionAfterSuccessUpload) === 'function') {
                            callbackFunctionAfterSuccessUpload(thisobj);
                        }
                    }
                }
            },
            error: function(jqXHR) {
                var response = jqXHR.responseJSON;
                console.log(response);
                if (typeof(response) !== 'undefined' && typeof(response.form_result) !== 'undefined' && typeof(response.form_result_text) !== 'undefined') {
                    thisobj.set('isAlert', true);
                    thisobj.set('alertMessage', response.form_result_text);
                    thisobj.set('alertStatus', response.form_result);
                }
            }
        });
    },

    // this is method that will be call back from other method. so, it needs `thisobj` to work properly.
    editSaveDoUpdate(thisobj) {
        console.log('do update to db');
        var formData = $('#form-edit').serialize();

        $.ajax({
            url: config.APP.phpApiUrl+'/edit/'+thisobj.get('id'),
            type: 'PATCH',
            data: formData,
            dataType: 'json',
            success: function(response) {
                console.log(response);
                if (typeof(response) !== 'undefined' && typeof(response.form_result) !== 'undefined' && typeof(response.form_result_text) !== 'undefined') {
                    thisobj.set('isAlert', true);
                    thisobj.set('alertMessage', response.form_result_text);
                    thisobj.set('alertStatus', response.form_result);
                }

                if (typeof(response) !== 'undefined') {
                    if (typeof(response.result) !== 'undefined' && response.result === true) {
                        $('#file').val('');
                        thisobj.send('doReload');
                    }
                }
            },
            error: function(jqXHR) {
                var response = jqXHR.responseJSON;
                console.log(response);
                if (typeof(response) !== 'undefined' && typeof(response.form_result) !== 'undefined' && typeof(response.form_result_text) !== 'undefined') {
                    thisobj.set('isAlert', true);
                    thisobj.set('alertMessage', response.form_result_text);
                    thisobj.set('alertStatus', response.form_result);
                }
            }
        });
    },

    getId: Ember.computed('editId', function() {
        return this.get('editId');
    }),

    id: 0,

    isAlert: false,

    isEditing: false,

    setDbDataToValues() {
        var dbData = this.get('dbData');
        if (dbData) {
            this.setProperties(Ember.getProperties(dbData, 'name', 'file', 'create', 'update'));
        }
    },
});
