<?php


namespace Modules\Home\Controllers;

use \Psr\Http\Message\ServerRequestInterface;
use \Psr\Http\Message\ResponseInterface;

/**
 * Home main controller.
 *
 * @author Vee W.
 */
class IndexController extends \Modules\Core\Controllers\BaseController
{


    public function addAction(ServerRequestInterface $request, ResponseInterface $response)
    {
        $form_validate = false;

        $data['name'] = filter_input(INPUT_POST, 'name', FILTER_SANITIZE_STRING);
        if (trim($data['name']) == null) {
            $form_validate = false;
            $output['result'] = false;
            $output['form_result'] = 'error';
            $output['form_result_text'] = 'Please enter name.';
            $response = $response->withStatus(400);
        } else {
            $form_validate = true;
        }

        // prepare move uploaded file. (if form validate passed from above.)
        if (isset($form_validate) && $form_validate === true && isset($_FILES['file']['name']) && $_FILES['file']['name'] != null) {
            $Upload = new \Rundiz\Upload\Upload('file');
            $Upload->move_uploaded_to = STORAGE_PATH . DIRECTORY_SEPARATOR . 'uploaded';
            $Upload->allowed_file_extensions = ['jpg', 'jpeg'];
            $Upload->max_file_size = 1048576;
            $Upload->web_safe_file_name = true;
            $Upload->upload();
            $uploaded_data = $Upload->getUploadedData();
            if (is_array($Upload->error_messages) && !empty($Upload->error_messages)) {
                $form_validate = false;
                $output['result'] = false;
                $output['form_result'] = 'error';
                $upload_error = '';
                foreach ($Upload->error_messages as $error_message) {
                    $upload_error .= '<p>' . $error_message . '</p>';
                }
                $output['form_result_text'] = $upload_error;
                $response = $response->withStatus(400);
                unset($upload_error);
            } else {
                $form_validate = true;
                $data['file'] = 'Storage/uploaded/' . $uploaded_data[0]['new_name'];
            }
        }

        // insert to db.
        if (isset($form_validate) && $form_validate === true) {
            $inserted = $this->Db->Conn->insert('`' . $this->Db->getTableName('table') . '`', $data);
            if ($inserted > 0) {
                $output['id'] = $this->Db->Conn->lastInsertId();
                $output['result'] = true;
                $output['form_result'] = 'success';
                $output['form_result_text'] = 'Added successfully.';
                $response = $response->withStatus(201);
            } else {
                $output['result'] = false;
                $output['form_result'] = 'error';
                $output['form_result_text'] = 'Unable to save data to db.';
                $response = $response->withStatus(500);
            }
        } 

        return $this->response(
            $request,
            $response,
            (isset($output) ? $output : [])
        );
    }// addAction


    public function deleteItemsAction(ServerRequestInterface $request, ResponseInterface $response)
    {
        $_DELETE = [];
        parse_str(file_get_contents('php://input'), $_DELETE);

        $ids = (isset($_DELETE['id']) ? $_DELETE['id'] : []);
        $deleted_count = 0;

        if (is_array($ids)) {
            foreach ($ids as $id) {
                // remove uploaded file.
                $sql = 'SELECT * FROM `' . $this->Db->getTableName('table') . '` WHERE `id` = :id';
                $stmt = $this->Db->Conn->prepare($sql);
                $stmt->bindValue(':id', $id);
                $stmt->execute();
                $row = $stmt->fetch();
                unset($sql, $stmt);
                if ($row != null) {
                    if (is_file($row->file) && is_writable($row->file)) {
                        unlink($row->file);
                    }
                    // delete record
                    $this->Db->Conn->delete('`' . $this->Db->getTableName('table') . '`', ['id' => $id]);
                    $deleted_count++;
                }
                unset($row);
            }
            unset($id);
        }

        if ($deleted_count <= 0) {
            $output['result'] = false;
            $output['form_result'] = 'error';
            $output['form_result_text'] = 'Please select at least one item to delete.';
            $response = $response->withStatus(404);
        } elseif (count($ids) > $deleted_count) {
            $output['result'] = false;
            $output['form_result'] = 'warn';
            $output['form_result_text'] = 'Delete items successfully but some items are not currently exists before it got delete.';
            $response = $response->withStatus(400);
        } else {
            $output['result'] = true;
            $output['form_result'] = 'success';
            $output['form_result_text'] = 'The selected items has been deleted.';
            $response = $response->withStatus(200);
        }

        unset($_DELETE, $deleted_count, $ids);

        return $this->response(
            $request,
            $response,
            (isset($output) ? $output : [])
        );
    }// deleteItemsAction


    public function editAction(ServerRequestInterface $request, ResponseInterface $response, $args)
    {
        $_PATCH = [];
        parse_str(file_get_contents('php://input'), $_PATCH);

        $id = (isset($args['id']) ? $args['id'] : 0);
        if (filter_var($id, FILTER_VALIDATE_INT) === false) {
            $id = 0;
        }

        $data['name'] = filter_var((isset($_PATCH['name']) ? $_PATCH['name'] : ''), FILTER_SANITIZE_STRING);
        if (trim($data['name']) == null) {
            $form_validate = false;
            $output['result'] = false;
            $output['form_result'] = 'error';
            $output['form_result_text'] = 'Please enter name.';
            $response = $response->withStatus(400);
        } else {
            $form_validate = true;
        }
        $data['`update`'] = date('Y-m-d H:i:s');

        // check exists data in db.
        $sql = 'SELECT * FROM `' . $this->Db->getTableName('table') . '` WHERE `id` = :id';
        $stmt = $this->Db->Conn->prepare($sql);
        $stmt->bindParam(':id', $id);
        $stmt->execute();
        $row = $stmt->fetch();
        unset($sql, $stmt);
        if ($row == null) {
            $form_validate = false;
            $output['result'] = false;
            $output['form_result'] = 'error';
            $output['form_result_text'] = 'Could not found editing id.';
            $output['id'] = $id;
            $response = $response->withStatus(404);
        }

        unset($row);

        // update to db.
        if (isset($form_validate) && $form_validate === true) {
            $updated = $this->Db->Conn->update('`' . $this->Db->getTableName('table') . '`', $data, ['id' => $id]);
            if ($updated >= 0) {
                $output['id'] = $id;
                $output['result'] = true;
                $output['form_result'] = 'success';
                $output['form_result_text'] = 'Updated successfully.';
                $response = $response->withStatus(200);
            } else {
                $output['result'] = false;
                $output['form_result'] = 'error';
                $output['form_result_text'] = 'Unable to save data to db.';
                $response = $response->withStatus(500);
            }
        }

        unset($_PATCH);

        return $this->response(
            $request,
            $response,
            (isset($output) ? $output : [])
        );
    }// editAction
    
    
    public function editUploadAction(ServerRequestInterface $request, ResponseInterface $response, $args)
    {
        $id = (isset($args['id']) ? $args['id'] : 0);
        if (filter_var($id, FILTER_VALIDATE_INT) === false) {
            $id = 0;
        }

        // check exists data in db.
        $sql = 'SELECT * FROM `' . $this->Db->getTableName('table') . '` WHERE `id` = :id';
        $stmt = $this->Db->Conn->prepare($sql);
        $stmt->bindParam(':id', $id);
        $stmt->execute();
        $row = $stmt->fetch();
        unset($sql, $stmt);
        if ($row == null) {
            $form_validate = false;
            $output['result'] = false;
            $output['form_result'] = 'error';
            $output['form_result_text'] = 'Could not found editing id.';
            $output['id'] = $id;
            $response = $response->withStatus(404);
        } else {
            $form_validate = true;
        }

        if (isset($form_validate) && $form_validate === true && isset($_FILES['file']['name']) && $_FILES['file']['name'] != null) {
            $Upload = new \Rundiz\Upload\Upload('file');
            $Upload->move_uploaded_to = STORAGE_PATH . DIRECTORY_SEPARATOR . 'uploaded';
            $Upload->allowed_file_extensions = ['jpg', 'jpeg'];
            $Upload->max_file_size = 1048576;
            $Upload->web_safe_file_name = true;
            $Upload->upload();
            $uploaded_data = $Upload->getUploadedData();
            if (is_array($Upload->error_messages) && !empty($Upload->error_messages)) {
                $form_validate = false;
                $output['result'] = false;
                $output['form_result'] = 'error';
                $upload_error = '';
                foreach ($Upload->error_messages as $error_message) {
                    $upload_error .= '<p>' . $error_message . '</p>';
                }
                $output['form_result_text'] = $upload_error;
                $response = $response->withStatus(400);
                unset($upload_error);
            } else {
                $form_validate = true;
                $data['file'] = 'Storage/uploaded/' . $uploaded_data[0]['new_name'];
                $data['`update`'] = date('Y-m-d H:i:s');
                // delete previous uploaded
                if (isset($row)) {
                    if (is_file($row->file) && is_writable($row->file)) {
                        unlink($row->file);
                    }
                }
                // update uploaded field
                $this->Db->Conn->update('`' . $this->Db->getTableName('table') . '`', $data, ['id' => $id]);
                $output['result'] = true;
                $output['form_result'] = 'success';
                $output['form_result_text'] = 'Upload successfully.';
                $output['id'] = $id;
                $response = $response->withStatus(200);
                unset($data);
            }
            unset($Upload, $uploaded_data);
        } else {
            $output['result'] = false;
            $output['form_result'] = 'error';
            $output['form_result_text'] = 'You did not upload any file.';
            $output['id'] = $id;
            $response = $response->withStatus(400);
        }

        unset($form_validate, $id, $row);

        return $this->response(
            $request,
            $response,
            (isset($output) ? $output : [])
        );
    }// editUploadAction


    public function getDataAction(ServerRequestInterface $request, ResponseInterface $response)
    {
        $id = filter_input(INPUT_GET, 'id', FILTER_SANITIZE_NUMBER_INT);
        if (filter_var($id, FILTER_VALIDATE_INT) === false) {
            $id = 0;
        }

        $sql = 'SELECT * FROM `' . $this->Db->getTableName('table') . '` WHERE `id` = :id';
        $stmt = $this->Db->Conn->prepare($sql);
        $stmt->bindParam(':id', $id);
        $stmt->execute();
        $output['result'] = $stmt->fetch();
        unset($id, $sql, $stmt);

        return $this->response(
            $request,
            $response,
            (isset($output) ? $output : [])
        );
    }// getDataAction


    public function indexAction(ServerRequestInterface $request, ResponseInterface $response)
    {
        $output['_test_hello'] = 'world';

        // list items in db.
        $sql = 'SELECT * FROM `' . $this->Db->getTableName('table') . '` ORDER BY `id` ASC';
        $stmt = $this->Db->prepare($sql);
        $stmt->execute();
        $output['result'] = $stmt->fetchAll();
        unset($sql, $stmt);

        // this is home page. just send user to /admin page.
        return $this->response(
            $request, 
            $response
                ->withHeader('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0')
                ->withAddedHeader('Cache-Control', 'post-check=0, pre-check=0')
                ->withHeader('Pragma', 'no-cache')
                ->withHeader('Expires', 'Sat, 26 Jul 1997 05:00:00 GMT'),
            (isset($output) ? $output : [])
        );
            //->withStatus(301)
            //->withHeader('Location', 'admin');
    }// indexAction


}
