<?php
/* 
 * 
 * @author Vee W.
 * @license http://opensource.org/licenses/MIT
 * 
 */


namespace System\Libraries\Db;

class Logger implements \Doctrine\DBAL\Logging\SQLLogger
{


    /**
     * Interface for SQL loggers.
     *
     * @link   www.doctrine-project.org
     * @link http://stackoverflow.com/questions/210564/getting-raw-sql-query-string-from-pdo-prepared-statements#answer-8403150 the query build from params.
     * @author Benjamin Eberlei <kontakt@beberlei.de>
     * @author Guilherme Blanco <guilhermeblanco@hotmail.com>
     * @author Jonathan Wage <jonwage@gmail.com>
     * @author Roman Borschel <roman@code-factory.org>
     */
    public function startQuery($sql, array $params = null, array $types = null)
    {
        $keys = [];
        $values = $params;
        $values_limit = [];

        $words_repeated = array_count_values(str_word_count($sql, 1, ':_'));

        if (is_array($params)) {
            foreach ($params as $key => $val) {
                if (is_string($key)) {
                    $keys[] = '/:' . $key . '/';
                    $values_limit[$key] = (isset($words_repeated[':'.$key]) ? intval($words_repeated[':'.$key]) : 1);
                } else {
                    $keys[] = '/[?]/';
                    $values_limit = [];
                }

                if (is_string($val)) {
                    $values[$key] = "'" . $val . "'";
                }
                if (is_array($val)) {
                    $values[$key] = "'" . implode("','", $val) . "'";
                }
                if (is_null($val)) {
                    $values[$key] = 'NULL';
                }
            }// endforeach;
            unset($key, $val);
        }

        if (is_array($values)) {
            foreach ($values as $key => $val) {
                if (isset($values_limit[$key])) {
                    $sql = preg_replace(['/:'.$key.'/'], [$val], $sql, $values_limit[$key], $count);
                } else {
                    $sql = preg_replace(['/:'.$key.'/'], [$val], $sql, 1, $count);
                }
            }
            unset($key, $val);
        } else {
            $sql = preg_replace($keys, $values, $sql, 1, $count);
        }
        unset($keys, $values, $values_limit, $words_repeated);

        if (defined('APP_ENV') && (APP_ENV == 'test' || APP_ENV == 'development')) {
            if (!file_exists(STORAGE_PATH)) {
                mkdir(STORAGE_PATH);
            }
            if (!file_exists(STORAGE_PATH . DIRECTORY_SEPARATOR . 'log')) {
                mkdir(STORAGE_PATH . DIRECTORY_SEPARATOR . 'log');
            }
            $log_file_path = STORAGE_PATH . DIRECTORY_SEPARATOR . 'log' . DIRECTORY_SEPARATOR . 'dbal_sql_query_log.log';
            if (file_exists($log_file_path) && is_file($log_file_path) && is_writable($log_file_path) && filesize($log_file_path) > 5242880) {
                // log file is too large. (limit to 5 MB, over this then delete it.)
                unlink($log_file_path);
            }// endif log file exists and too large.
            // add backtrace to log.
            $traces = debug_backtrace(DEBUG_BACKTRACE_PROVIDE_OBJECT, 2);
            if (is_array($traces) && !empty($traces)) {
                $sql .= "\r\n";
                $sql .= '  Date/time: ' . date('Y-m-d H:i:s') . "\r\n";
                foreach ($traces as $trace_item) {
                    if (is_array($trace_item) && array_key_exists('file', $trace_item) && array_key_exists('line', $trace_item)) {
                        $sql .= '  ' . $trace_item['file'] . ':' . $trace_item['line'] . "\r\n";
                    }
                }// endforeach;
                unset($trace_item);
            }
            unset($traces);
            file_put_contents($log_file_path, $sql . "\r\n", FILE_APPEND);
            unset($log_file_path);
        }
        unset($sql);
    }// startQuery


    /**
     * {@inheritdoc}
     */
    public function stopQuery()
    {
    }// stopQuery


}